# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2017-06-05 15:07
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='BankCodeDE',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('bank_code', models.CharField(max_length=8, verbose_name='Bank code')),
                ('payment_service_provider', models.CharField(choices=[(b'1', 'Yes'), (b'2', 'No')], default=b'2', help_text='Merkmal, ob bankleitzahlfu\u0308hrender Zahlungsdienstleister (\u201e1\u201c) oder nicht (\u201e2\u201c)', max_length=1, verbose_name='Payment service provider')),
                ('description', models.CharField(help_text='Bezeichnung des Zahlungsdienstleisters (ohne Rechtsform)', max_length=58, verbose_name='Description')),
                ('zip_code', models.CharField(max_length=5, verbose_name='Zip code')),
                ('city', models.CharField(max_length=35, verbose_name='City')),
                ('short_description', models.CharField(help_text=b'Kurzbezeichnung des Zahlungsdienstleisters mit Ort (ohne Rechtsform)', max_length=27, verbose_name='Short description')),
                ('pan', models.CharField(help_text='Institutsnummer f\xfcr PAN', max_length=5, verbose_name='PAN')),
                ('bic', models.CharField(help_text='Business Identifier Code', max_length=11, verbose_name='BIC')),
                ('check_digit_method', models.CharField(help_text='Kennzeichen f\xfcr Pr\xfcfzifferberechnungsmethode', max_length=2, verbose_name='Check digit method')),
                ('dataset_number', models.CharField(help_text='Nummer des Datensatzes', max_length=6, verbose_name='Dataset number')),
                ('indicator_changed', models.CharField(choices=[(b'A', 'New'), (b'D', 'Deleted'), (b'M', 'Changed'), (b'U', 'Unchanged')], default=b'A', help_text='\xc4nderungskennzeichen \u201eA\u201c (Addition) f\xfcr neue, \u201eD\u201c (Deletion) f\xfcr gel\xf6schte, \u201eU\u201c(Unchanged) f\xfcr unver\xe4nderte und \u201eM\u201c (Modified) f\xfcr ver\xe4nderte Datens\xe4tze', max_length=1, verbose_name='Indicator changed')),
                ('indicator_deleted', models.CharField(choices=[(b'0', 'No declaration'), (b'1', 'Declarted for deletion')], default=b'0', help_text='Hinweis auf eine beabsichtigte Bankleitzahll\xf6schung', max_length=1, verbose_name='Indicator deleted')),
                ('succession_bank_code', models.CharField(help_text='Hinweis auf Nachfolge-Bankleitzahl', max_length=8, verbose_name='Succession bank code')),
                ('iban_rule', models.CharField(help_text='Kennzeichen f\xfcr die IBAN-Regel', max_length=6, verbose_name='IBAN Rule')),
            ],
            options={
                'ordering': ('bank_code',),
                'verbose_name': 'German Bank Code',
                'verbose_name_plural': 'German Bank Codes',
            },
        ),
    ]



# -*- coding: utf-8 -*-
import re
from . import models
from . import operations


def validate_bic(bic):

    if not re.match('^[a-zA-Z0-9_]{,11}$', bic):
        return False

    bic = bic.upper()
    count = len(models.BankCodeDE.objects.filter(bic=bic))

    return True if count > 0 else False


def validate_iban(iban):

    iban = iban.upper()
    if not re.match('^[A-Z]{2}[0-9]{20}$', iban):
        return False

    checksum = int("%s%d%s" % (
        iban[4:],
        operations.get_numeric_country_code(iban[:2]),
        iban[2:4],
    ))

    mod_result = checksum % 97

    return mod_result == 1


def get_iban_by_bank_account(account_number, bank_number, country='DE'):

    if country != 'DE':
        raise ValueError('This version only supports german iban number generation')

    if not re.match('^[0-9]{6,10}$', account_number):
        raise ValueError('Unknown account number type')

    if not re.match('^[0-9]{8}$', bank_number):
        raise ValueError('Unknown bank number type')

    account_number = "%010d" % int(account_number)
    bban = "%s%s" % (bank_number, account_number)
    country_code = operations.get_numeric_country_code(country)
    country_code_ex = "%s00" % country_code
    checksum = int("%s%s" % (bban, country_code_ex))
    mod_result = checksum % 97
    mod_checksum = "%02d" % (98 - mod_result)

    iban = "%s%s%s" % (country, mod_checksum, bban)

    return iban



# -*- coding: utf-8 -*-
import re


def get_numeric_country_code(iso2_country_code):

    if not re.match('^[A-Z]{2}$', iso2_country_code):
        return False

    alphabet = list(map(chr, range(ord('A'), ord('Z') + 1)))
    first_letter_code = alphabet.index(iso2_country_code[:1]) + 10
    second_letter_code = alphabet.index(iso2_country_code[1:]) + 10

    return int('%d%d' % (first_letter_code, second_letter_code))



# -*- coding: utf-8 -*-
from django.db import models
from django.utils.translation import ugettext_lazy as _


class BankCodeDE(models.Model):

    PSP_YES = '1'
    PSP_NO = '2'
    PSP_CHOICES = (
        (PSP_YES, _('Yes')),
        (PSP_NO, _('No')),
    )

    AK_NEW = 'A'
    AK_DELETED = 'D'
    AK_CHANGED = 'M'
    AK_UNCHANGED = 'U'
    AK_CHOICES = (
        (AK_NEW, _('New')),
        (AK_DELETED, _('Deleted')),
        (AK_CHANGED, _('Changed')),
        (AK_UNCHANGED, _('Unchanged')),
    )

    BBL_NS = '0'
    BBL_DP = '1'
    BBL_CHOICES = (
        (BBL_NS, _('No declaration')),
        (BBL_DP, _('Declarted for deletion')),
    )

    bank_code = models.CharField(max_length=8, verbose_name=_('Bank code'))
    payment_service_provider = models.CharField(max_length=1, choices=PSP_CHOICES, default=PSP_NO, verbose_name=_('Payment service provider'), help_text=u'Merkmal, ob bankleitzahlführender Zahlungsdienstleister („1“) oder nicht („2“)')
    description = models.CharField(max_length=58, verbose_name=_('Description'), help_text=u'Bezeichnung des Zahlungsdienstleisters (ohne Rechtsform)')
    zip_code = models.CharField(max_length=5, verbose_name=_('Zip code'))
    city = models.CharField(max_length=35, verbose_name=_('City'))
    short_description = models.CharField(max_length=27, verbose_name=_('Short description'), help_text='Kurzbezeichnung des Zahlungsdienstleisters mit Ort (ohne Rechtsform)')
    pan = models.CharField(max_length=5, verbose_name=_('PAN'), help_text=u'Institutsnummer für PAN')
    bic = models.CharField(max_length=11, verbose_name=_('BIC'), help_text=u'Business Identifier Code')
    check_digit_method = models.CharField(max_length=2, verbose_name=_('Check digit method'), help_text=u'Kennzeichen für Prüfzifferberechnungsmethode')
    dataset_number = models.CharField(max_length=6, verbose_name=_('Dataset number'), help_text=u'Nummer des Datensatzes')
    indicator_changed = models.CharField(max_length=1, verbose_name=_('Indicator changed'), choices=AK_CHOICES, default=AK_NEW, help_text=u'Änderungskennzeichen „A“ (Addition) für neue, „D“ (Deletion) für gelöschte, „U“(Unchanged) für unveränderte und „M“ (Modified) für veränderte Datensätze')
    indicator_deleted = models.CharField(max_length=1, verbose_name=_('Indicator deleted'), choices=BBL_CHOICES, default=BBL_NS, help_text=u'Hinweis auf eine beabsichtigte Bankleitzahllöschung')
    succession_bank_code = models.CharField(max_length=8, verbose_name=_('Succession bank code'), help_text=u'Hinweis auf Nachfolge-Bankleitzahl')
    iban_rule = models.CharField(max_length=6, verbose_name=_('IBAN Rule'), help_text=_(u'Kennzeichen für die IBAN-Regel'))

    class Meta:
        verbose_name = _('German Bank Code')
        verbose_name_plural = _('German Bank Codes')
        ordering = ('bank_code', )

    def __unicode__(self):
        return self.description



# -*- coding: utf-8 -*-
from django.contrib import admin
from ibantools.models import BankCodeDE


@admin.register(BankCodeDE)
class BankCodeDEAdmin(admin.ModelAdmin):

    list_display = (
        'bank_code',
        'description',
        'city',
        'check_digit_method',
        'bic',
    )
    list_filter = (
        'payment_service_provider',
        'indicator_changed',
        'indicator_deleted',
    )
    search_fields = (
        'description',
        'city',
        'bank_code',
        'bic',
        'zip_code',
    )

    actions_selection_counter = True



# -*- coding: utf-8 -*-
import csv
from django.core.management.base import BaseCommand
from ...models import BankCodeDE


class Command(BaseCommand):
    help = 'Importiert Bankleitzaheln über eine von der Bundesbank gestellte CSV Datei'

    def add_arguments(self, parser):
        parser.add_argument('csvfile', type=str)

    def handle(self, *args, **options):

        BankCodeDE.objects.all().delete()
        csv.register_dialect('bundesbank', delimiter='|', quoting=csv.QUOTE_NONE)

        with open(options['csvfile'], 'rb') as csvfile:
            reader = csv.reader(csvfile, 'bundesbank')
            for line in reader:
                data = unicode(''.join(line), "ISO-8859-1")
                satz = BankCodeDE.objects.create()

                satz.bank_code = unicode(data[0:8]).strip()
                satz.payment_service_provider = unicode(data[8:9]).strip()
                satz.description = unicode(data[9:67]).strip()
                satz.zip_code = unicode(data[67:72]).strip()
                satz.city = unicode(data[72:107]).strip()
                satz.short_description = unicode(data[107:134]).strip()
                satz.pan = unicode(data[134:139]).strip()
                satz.bic = unicode(data[139:150]).strip()
                satz.check_digit_method = unicode(data[150:152]).strip()
                satz.dataset_number = unicode(data[152:158]).strip()
                satz.indicator_changed = unicode(data[158:159]).strip()
                satz.indicator_deleted = unicode(data[159:160]).strip()
                satz.succession_bank_code = unicode(data[160:168]).strip()
                satz.iban_rule = unicode(data[168:174]).strip()

                satz.save()

            print "Import Done"



import os

from setuptools import setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-ibantools',
    version='0.4.1',
    packages=['ibantools'],
    include_package_data=True,
    license='BSD License',
    description='A simple IBAN/Bic Helper app',
    long_description=README,
    url='https://github.com/nadobit/django-ibantools',
    author='Bastian Probian',
    author_email='contmp@me.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.11',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
    ],
)



# -*- coding: utf-8 -*-
from django.test import TestCase
from ibantools import models
from ibantools import utils


class ValidateBicTestCase(TestCase):

    def setUp(self):
        models.BankCodeDE.objects.create(
            bic='MARKDEF1100'
        )

    def test_valid_bic_number(self):
        self.assertTrue(utils.validate_bic('MARKDEF1100'))

    def test_invalid_bic_numbers(self):
        self.assertFalse(utils.validate_bic('MARKDEF1101'))
        self.assertFalse(utils.validate_bic('as!d'))
        self.assertFalse(utils.validate_bic('asd'))


class ValidateIBANTestCase(TestCase):

    def test_valid_iban_number(self):
        self.assertTrue(utils.validate_iban('DE08700901001234567890'))

    def test_invalid_iban_number(self):
        self.assertFalse(utils.validate_iban('DE08700901001234567891'))
        self.assertFalse(utils.validate_iban('asd!'))
        self.assertFalse(utils.validate_iban('asd'))


class ValidateIBANGenerationTestCase(TestCase):

    def test_success_generation(self):
        self.assertEquals(
            utils.get_iban_by_bank_account(
                account_number='70001506',
                bank_number='70000000'),
            'DE05700000000070001506'
        )

    def test_unsupported_country(self):
        with self.assertRaises(ValueError):
            utils.get_iban_by_bank_account(
                country='US',
                account_number='70001506',
                bank_number='70000000'
            )

    def test_invalid_account_numbers(self):
        with self.assertRaises(ValueError):
            utils.get_iban_by_bank_account(
                account_number='!123123132123',
                bank_number='70000000'
            )

        with self.assertRaises(ValueError):
            utils.get_iban_by_bank_account(
                account_number='123',
                bank_number='70000000'
            )

    def test_invalid_bank_numbers(self):
        with self.assertRaises(ValueError):
            utils.get_iban_by_bank_account(
                account_number='70001506',
                bank_number='!70000000'
            )

        with self.assertRaises(ValueError):
            utils.get_iban_by_bank_account(
                account_number='70001506',
                bank_number='1'
            )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class IbantoolsConfig(AppConfig):
    name = 'ibantools'
    verbose_name = _('IBAN Tools')
